from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.contrib.syndication.feeds import Feed

from models import FTPLog, FTP_MODE

FTP_LOG_FEED_NUM = getattr(settings, 'FTP_LOG_FEED_NUM', 100)

class FTPLogFeed(Feed):
    title = _('FTPD log')
    description = _('FTPD log feeds.')
    log_model = FTPLog
    mode = None
    num_output = FTP_LOG_FEED_NUM
    link = ''
    item_link = ''
    title_template = 'feeds/ftplog_title.html'
    description_template = 'feeds/ftplog_description.html'

    def items(self):
        logs = self.log_model.objects.all()
        if self.mode:
            logs = logs.filter(mode=self.mode)
        return logs[:self.num_output]

    def item_pubdate(self, item):
        return item.created_at

    def item_author_name(self, item):
        return item.user



from django.contrib import admin

from models import FTPUserGroup, FTPUser, FTPLog

class FTPUserGroupAdmin(admin.ModelAdmin):
    list_display = ('name', 'permission',)
    search_fields = ('name', 'permission',)

class FTPUserAdmin(admin.ModelAdmin):
    list_display = ('user', 'usergroup', 'last_login')
    search_fields = ('user', 'usergroup', 'last_login')

class FTPLogAdmin(admin.ModelAdmin):
    list_display = ('filename', 'mode', 'user', 'remote_path', 'created_at',)
    list_filter = ('mode', 'user', 'created_at')
    search_fields = ('remote_path', 'local_path',)

admin.site.register(FTPUserGroup, FTPUserGroupAdmin)
admin.site.register(FTPUser, FTPUserAdmin)
admin.site.register(FTPLog, FTPLogAdmin)



import os
import sys
import time
import stat

try:
    import pwd
    import grp
except ImportError:
    pwd = grp = None

from django.conf import settings
from django.utils.encoding import smart_unicode

try:
    from pyftpdlib.ftpserver import FTPHandler, AbstractedFS
except ImportError:
    print 'Please install Python FTP server library(pyftpdlib).\n' \
          'http://code.google.com/p/pyftpdlib/'
    exit()

from unboxftpd.models import FTPUser, FTPLog

class ExtendedFS(AbstractedFS):

    def format_mlsx(self, basedir, listing, perms, facts, ignore_err=True):
        """
        optimize encoding
        this method called by MLSD command.
        """
        permdir = ''.join([x for x in perms if x not in 'arw'])
        permfile = ''.join([x for x in perms if x not in 'celmp'])
        if ('w' in perms) or ('a' in perms) or ('f' in perms):
            permdir += 'c'
        if 'd' in perms:
            permdir += 'p'
        type = size = perm = modify = create = unique = mode = uid = gid = ""
        for basename in listing:
            file = os.path.join(basedir, basename)
            try:
                st = self.stat(file)
            except OSError:
                if ignore_err:
                    continue
                raise
            # type + perm
            if stat.S_ISDIR(st.st_mode):
                if 'type' in facts:
                    if basename == '.':
                        type = 'type=cdir;'
                    elif basename == '..':
                        type = 'type=pdir;'
                    else:
                        type = 'type=dir;'
                if 'perm' in facts:
                    perm = 'perm=%s;' %permdir
            else:
                if 'type' in facts:
                    type = 'type=file;'
                if 'perm' in facts:
                    perm = 'perm=%s;' %permfile
            if 'size' in facts:
                size = 'size=%s;' %st.st_size  # file size
            # last modification time
            if 'modify' in facts:
                try:
                    modify = 'modify=%s;' %time.strftime("%Y%m%d%H%M%S",
                                           time.localtime(st.st_mtime))
                # it could be raised if last mtime happens to be too old
                # (prior to year 1900)
                except ValueError:
                    modify = ""
            if 'create' in facts:
                # on Windows we can provide also the creation time
                try:
                    create = 'create=%s;' %time.strftime("%Y%m%d%H%M%S",
                                           time.localtime(st.st_ctime))
                except ValueError:
                    create = ""
            # UNIX only
            if 'unix.mode' in facts:
                mode = 'unix.mode=%s;' %oct(st.st_mode & 0777)
            if 'unix.uid' in facts:
                uid = 'unix.uid=%s;' %st.st_uid
            if 'unix.gid' in facts:
                gid = 'unix.gid=%s;' %st.st_gid
            # We provide unique fact (see RFC-3659, chapter 7.5.2) on
            # posix platforms only; we get it by mixing st_dev and
            # st_ino values which should be enough for granting an
            # uniqueness for the file listed.
            # The same approach is used by pure-ftpd.
            # Implementors who want to provide unique fact on other
            # platforms should use some platform-specific method (e.g.
            # on Windows NTFS filesystems MTF records could be used).
            if 'unique' in facts:
                unique = "unique=%x%x;" %(st.st_dev, st.st_ino)

            #yield "%s%s%s%s%s%s%s%s%s %s\r\n" %(type, size, perm, modify, create,
            #                                    mode, uid, gid, unique, basename)

            # opts encoding
            line = u"%s%s%s%s%s%s%s%s%s %s\r\n" %(type, size, perm, modify, create,
                                                 mode, uid, gid, unique, smart_unicode(basename))
            yield line.encode('utf8')

class Authorizer(object):
    """
    Authorizer class by FTPUser model.
    """
    user_class = FTPUser

    def get_user(self, username):
        return self.user_class.get_user(username)

    def validate_authentication(self, username, password):
        user = self.get_user(username)
        return not not (user and user.check_password(password))

    def impersonate_user(self, username, password):
        pass    

    def terminate_impersonation(self):
        pass

    def has_user(self, username):
        return self.get_user(username)

    def get_home_dir(self, username):
        user = self.get_user(username)
        # return ascii
        return user and str(user.home_directory)

    def get_msg_login(self, username):
        user = self.get_user(username)
        if user:
            user.login_update()
        return 'welcome.'

    def get_msg_quit(self, username):
        return 'good bye.'

    def has_perm(self, username, perm, path=None):
        user = self.get_user(username)
        return user and user.has_perm(perm, path)

    def get_perms(self, username):
        user = self.get_user(username)
        return user and user.get_perms()

class LoggingHandler(object):
    log_class = FTPLog
    user_class = FTPUser

    def __call__(self, ftp_handler, line, mode):
        """
        logging handler
        """
        user = self.user_class.get_user(ftp_handler.username)

        # TODO detect or force convert
        #if ftp_handler.is_client_utf8:
        #    line = unicode(line, 'utf-8')

        remote_path = ftp_handler.fs.fs2ftp(line)

        if user:
            self.log_class.logging(
                user=user,
                mode=mode,
                filename=smart_unicode(os.path.basename(line)),
                remote_path=smart_unicode(remote_path),
                local_path=smart_unicode(line)
            )

class LoggingFTPHandler(FTPHandler):
    """
    FTPHandler with simple logging.
    """
    logging_handler = LoggingHandler
    is_client_utf8 = False
    default_encoding = 'UTF-8'

    def __init__(self, *args, **kwargs):
        FTPHandler.__init__(self, *args, **kwargs)
        if self.logging_handler:
            self._logging = LoggingHandler()
        if self.get_encoding() == 'UTF-8':
            self._extra_feats += ['UTF8']

    def ftp_STOR(self, line, mode='w'):
        """
        upload
        """
        FTPHandler.ftp_STOR(self, line, mode=mode)
        self.logging(line, 'STOR')

    def ftp_RETR(self, line):
        """
        download
        """
        FTPHandler.ftp_RETR(self, line)
        self.logging(line, 'RETR')

    def ftp_DELE(self, line):
        """
        delete
        """
        FTPHandler.ftp_DELE(self, line)
        self.logging(line, 'DELE')

    def ftp_OPTS(self, line):
        """
        opts for utf8 encoding.
        """
        if line.count(' ') == 1:
            cmd, arg = line.upper().split(' ')
            if cmd == 'UTF8' and arg == 'ON' and  \
                    self.get_encoding() == 'UTF-8':
                self.is_client_utf8 = True
                self.respond('200 OK.')
                return
        FTPHandler.ftp_OPTS(self, line)

    def get_encoding(self):
        return self.default_encoding

    def logging(self, line, mode):
        if self.logging_handler:
            try:
                self._logging(ftp_handler=self, line=line, mode=mode)
            except:
                if settings.DEBUG:
                    raise



import sys
import datetime
from optparse import make_option

from django.conf import settings
from django.core.management.base import BaseCommand
from django.utils.daemonize import become_daemon

try:
    from pyftpdlib.ftpserver import FTPServer
except ImportError:
    from unboxftpd.pyftpdlib.ftpserver import FTPServer

from unboxftpd.server import ExtendedFS, Authorizer, LoggingFTPHandler

FTPD_DEFAULT_SETTINGS = {
    'FTPD_ADDRESS': 'localhost',
    'FTPD_PORT': 21,
    'FTPD_LOG': None,
    'FTPD_ERROR_LOG': None,
}

def get_settings(name):
    return getattr(settings, name, FTPD_DEFAULT_SETTINGS[name])

class Command(BaseCommand):

    option_list = BaseCommand.option_list + (
        make_option('--daemonize', action='store_true',
                dest='daemonize', help='FTPD become daemon'),
        make_option('--no-logging', action='store_false',
                dest='no-logging', help='No logging')
    )
    args = '[optional port number, or ipaddr:port]'

    def handle(self, addrport='', *args, **options):
        # logging
        if options['no-logging']:
            LoggingFTPHandler.logging_handler = None
        LoggingFTPHandler.authorizer = Authorizer()
        LoggingFTPHandler.abstracted_fs = ExtendedFS

        # ftp port:addr
        # default localhost:21
        if addrport:
            addr, port = addrport.split(':')
        else:
            addr = get_settings('FTPD_ADDRESS')
            port = get_settings('FTPD_PORT')

        # create server instance
        ftpd = FTPServer((addr, port), LoggingFTPHandler)

        # daemonize
        if options['daemonize']:
            kwargs_daemon = {}
            if get_settings('FTPD_LOG'):
                kwargs_daemon['out_log'] = get_settings('FTPD_LOG')
            if get_settings('FTPD_ERROR_LOG'):
                kwargs_daemon['err_log'] = get_settings('FTPD_ERROR_LOG')

            become_daemon(**kwargs_daemon)

        sys.stdout.write('[%s]\nStarting UnboxFTPD\n' % \
                datetime.datetime.now().strftime('%Y%m/%d %H:%m:%S'))
        ftpd.serve_forever()



import datetime
from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ObjectDoesNotExist

FTP_MODE_UPLOAD = 0
FTP_MODE_DOWNLOAD = 1
FTP_MODE_DELETE = 2

FTP_MODE = (
    (FTP_MODE_UPLOAD, _('Upload')),
    (FTP_MODE_DOWNLOAD, _('Download')),
    (FTP_MODE_DELETE, _('Delete')),
)
FTP_COMMAND_MODE = {
    'STOR': FTP_MODE_UPLOAD,
    'RETR': FTP_MODE_DOWNLOAD,
    'DELE': FTP_MODE_DELETE,
}

class FTPUserGroup(models.Model):
    name = models.CharField(_('Group name'), null=False,
            blank=False, max_length=100)
    home_directory = models.CharField(_('Home directory'), null=False,
            blank=False, max_length=255)
    permission = models.CharField(_('Permission'),
            null=False, blank=False, max_length=8, default='elradfmw')

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = _('FTP user group')
        verbose_name_plural = _('FTP user groups')

def replace_special_name(self, path, ftpuser):
    """
    get_directory_handler
    replace special name.
    """
    path = path.replace('<username>', ftpuser.user.username)
    return path

class FTPUser(models.Model):
    user = models.OneToOneField(User, verbose_name=_('User'),
            null=False, blank=False)
    usergroup = models.ForeignKey(FTPUserGroup,
            verbose_name=_('FTP user group'), null=False, blank=False)
    last_login = models.DateTimeField(_('Last login'),
            default=datetime.datetime.now, editable=False)

    get_directory_handler = replace_special_name

    def __unicode__(self):
        try:
            user = self.user
        except:
            user = None
        return user and user.username

    def _get_home_directory(self):
        dir_path = self.usergroup.home_directory
        if self.get_directory_handler:
            return self.get_directory_handler(dir_path, self)
        return dir_path
    home_directory = property(_get_home_directory)

    def check_password(self, password):
        return self.user.check_password(password)

    def has_perm(self, perm, path):
        """
        permission test.
        """
        return perm in self.get_perms()

    def get_perms(self):
        """
        return permission string.
        """
        return self.usergroup.permission

    def login_update(self):
        self.last_login = datetime.datetime.now()

    @classmethod
    def get_user(cls, username):
        try:
            return cls.objects.get(user__username=username)
        except ObjectDoesNotExist:
            pass

    class Meta:
        verbose_name = _('FTP user')
        verbose_name_plural = _('FTP users')

class FTPLog(models.Model):
    user = models.ForeignKey(FTPUser, verbose_name=_('FTP user'))
    mode = models.IntegerField(_('Mode'), null=False,
            blank=False, choices=FTP_MODE)
    filename = models.CharField(_('Filename'), blank=True,
            max_length=255)
    remote_path = models.CharField(_('Remote path name'),
            blank=True, max_length=1024)
    local_path = models.CharField(_('Local path name'),
            blank=True, max_length=1024)
    created_at = models.DateTimeField(_('Created at'),
            blank=False, default=datetime.datetime.now, editable=False)

    def __unicode__(self):
        return self.filename

    @classmethod
    def logging(cls, *args, **kwargs):
        kwargs['mode'] = FTP_COMMAND_MODE[kwargs['mode']]
        cls.objects.create(*args, **kwargs)

    class Meta:
        verbose_name = _('FTP log')
        verbose_name_plural = _('FTP logs')
        ordering = ('-created_at',)    



#!/usr/bin/env python
from distutils.core import setup
import os

name = 'unboxftpd'
version = '0.1'

packages, data_files = [], []
root_dir = os.path.dirname(__file__)
if root_dir:
    os.chdir(root_dir)

for dirpath, dirnames, filenames in os.walk(name):
    for i, dirname in enumerate(dirnames):
        if dirname.startswith('.'): del dirnames[i]
    if '__init__.py' in filenames:
        pkg = dirpath.replace(os.path.sep, '.')
        if os.path.altsep:
            pkg = pkg.replace(os.path.altsep, '.')
        packages.append(pkg)
    elif filenames:
        prefix = dirpath[len(name + os.sep):]
        for f in filenames:
            data_files.append(os.path.join(prefix, f))

setup(name=name,
      version=version,
      description='A FTP server application for Django',
      long_description="unboxftpd add FTP server to your Django project easily.",
      author='Shinya Okano',
      author_email='tokibito@gmail.com',
      url='http://bitbucket.org/tokibito/unboxftpd/',
      download_url='http://bitbucket.org/tokibito/unboxftpd/get/0.1.gz',
      packages=packages,
      package_data={'unboxftpd': data_files},
      keywords=['ftp', 'server', 'ftpd', 'django', 'python'],
      classifiers=['Development Status :: 3 - Alpha',
                   'Environment :: Web Environment',
                   'Environment :: Console',
                   'Framework :: Django',
                   'License :: OSI Approved :: BSD License',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python',
                   'Topic :: Internet :: File Transfer Protocol (FTP)',
                   ],
      )
